#!/usr/bin/python3

from urllib.parse import urlencode
from urllib.request import Request, urlopen
import pytest
from StreamingSim import run_simulation_fixture


class TestSimulation(object):
    """
    A testing class used to group all the tests related to the simulation data
    """

    @pytest.mark.parametrize("query, expected_result", [
        ("SELECT count(*) FROM \"CLMCMetrics\".\"autogen\".\"cpu_usage\"", "{\"results\":[{\"statement_id\":0,\"series\":[{\"name\":\"cpu_usage\",\"columns\":[\"time\",\"count_cpu_active_time\",\"count_cpu_idle_time\",\"count_cpu_usage\"],\"values\":[[\"1970-01-01T00:00:00Z\",7200,7200,7200]]}]}]}"),
        ("SELECT count(*) FROM \"CLMCMetrics\".\"autogen\".\"ipendpoint_route\"", "{\"results\":[{\"statement_id\":0,\"series\":[{\"name\":\"ipendpoint_route\",\"columns\":[\"time\",\"count_http_requests_fqdn_m\",\"count_network_fqdn_latency\"],\"values\":[[\"1970-01-01T00:00:00Z\",7200,7200]]}]}]}"),
        ("SELECT count(*) FROM \"CLMCMetrics\".\"autogen\".\"mpegdash_service\"", "{\"results\":[{\"statement_id\":0,\"series\":[{\"name\":\"mpegdash_service\",\"columns\":[\"time\",\"count_avg_response_time\",\"count_peak_response_time\",\"count_requests\"],\"values\":[[\"1970-01-01T00:00:00Z\",7200,7200,7200]]}]}]}"),
        ("SELECT count(*) FROM \"CLMCMetrics\".\"autogen\".\"net_port_io\"", "{\"results\":[{\"statement_id\":0,\"series\":[{\"name\":\"net_port_io\",\"columns\":[\"time\",\"count_RX_BYTES_PORT_M\",\"count_TX_BYTES_PORT_M\"],\"values\":[[\"1970-01-01T00:00:00Z\",7200,7200]]}]}]}"),
        ("SELECT count(*) FROM \"CLMCMetrics\".\"autogen\".\"vm_res_alloc\"", "{\"results\":[{\"statement_id\":0,\"series\":[{\"name\":\"vm_res_alloc\",\"columns\":[\"time\",\"count_cpu\",\"count_memory\",\"count_storage\"],\"values\":[[\"1970-01-01T00:00:00Z\",12,12,12]]}]}]}")
    ])
    def test_simulation(self, query, expected_result, run_simulation_fixture):
        """
        This is the entry point of the test. This method will be found and executed when the module is ran using pytest

        :param query: the query to execute (value obtained from the pytest parameter decorator)
        :param expected_result: the result expected from executing the query (value obtained from the pytest parameter decorator)
        :param run_simulation_fixture: the imported fixture to use to generate the testing data - the return value of the fixture is not needed in this case
        """

        print("\n")  # prints a blank line for formatting purposes

        # pytest automatically goes through all queries under test, declared in the parameters decorator
        actual_result = self.send_query("http://localhost:8086", query)

        assert expected_result == actual_result, "Simulation test failure"

        print("Successfully passed test for the following query: {0}".format(query))

    @staticmethod
    def send_query(url, query):
        """
        An auxiliary static method to send a query to a url and retrieve the result

        :param url: the target url to which the query is sent to - a string containing a valid URL address
        :param query: the query to be executed on the given URL
        :return: the result of the executed query
        """

        query = urlencode({"q": query}).encode("ascii")
        request = Request("{0}/query".format(url), query)
        result = urlopen(request)

        return result.read().decode("utf-8").strip()
